#include <avr/io.h>			// for the input/output register
#include <avr/interrupt.h> 	
#include <util/delay.h>  	// for the _delay_ms
#include "RGB_setup.h"
#include "afficheurs.h"

#define PRESCALER 1024
#define TIME_SLOT 20
#define NB_TICK 113
#define BAUDRATE	103 // UBRR value for 9600


#define task_matrice 0
#define task_afficheurs 1


#define SAVE_CONTEXT()  \
	asm volatile ( \
		"push	r0				\n\t" \
		"in		r0, __SREG__	\n\t" \
		"cli					\n\t" \
		"push	r0				\n\t" \
        "push    r1             \n\t" \
        "push    r2             \n\t" \
        "push    r3             \n\t" \
        "push    r4             \n\t" \
        "push    r5             \n\t" \
        "push    r6             \n\t" \
        "push    r7             \n\t" \
        "push    r8             \n\t" \
        "push    r9             \n\t" \
        "push    r10             \n\t" \
        "push    r11             \n\t" \
        "push    r12             \n\t" \
        "push	 r13		     \n\t" \
        "push    r14             \n\t" \
        "push    r15             \n\t" \
        "push    r16             \n\t" \
        "push    r17             \n\t" \
        "push    r18             \n\t" \
        "push    r19             \n\t" \
        "push    r20             \n\t" \
        "push    r21             \n\t" \
        "push	 r22			 \n\t" \
        "push    r23             \n\t" \
        "push    r24             \n\t" \
        "push    r25             \n\t" \
        "push    r26             \n\t" \
        "push    r27             \n\t" \
        "push    r28             \n\t" \
        "push    r29             \n\t" \
        "push    r30             \n\t" \
        "push	 r31			 \n\t" \
            );

#define RESTORE_CONTEXT() \
	asm volatile ( \
        "pop	r31				\n\t" \
        "pop    r30             \n\t" \
        "pop    r29             \n\t" \
        "pop    r28             \n\t" \
        "pop    r27            \n\t" \
        "pop    r26             \n\t" \
        "pop    r25             \n\t" \
        "pop    r24             \n\t" \
        "pop    r23             \n\t" \
        "pop    r22             \n\t" \
        "pop    r21             \n\t" \
        "pop    r20             \n\t" \
        "pop    r19             \n\t" \
        "pop	 r18				\n\t" \
        "pop    r17             \n\t" \
        "pop    r16             \n\t" \
        "pop    r15             \n\t" \
        "pop    r14             \n\t" \
        "pop    r13             \n\t" \
        "pop    r12             \n\t" \
        "pop    r11             \n\t" \
        "pop    r10             \n\t" \
        "pop	 r9				\n\t" \
        "pop    r8             \n\t" \
        "pop    r7             \n\t" \
        "pop    r6             \n\t" \
        "pop    r5             \n\t" \
        "pop    r4             \n\t" \
        "pop    r3             \n\t" \
        "pop    r2             \n\t" \
        "pop    r1             \n\t" \
        "pop	 r0				\n\t" \
    );
    
typedef struct task_s {
    volatile uint16_t stack_pointer;
    volatile uint8_t state;
    volatile uint16_t task_id;
} task_t;

task_t tab_task[2];

int first=0;

int compteur;


void init_timer()
{
	TCCR1B |= _BV(WGM12); // CTC mode with value in OCR1A 
	TCCR1B |= _BV(CS12);  // CS12 = 1; CS11 = 0; CS10 =1 => CLK/1024 prescaler
	TCCR1B |= _BV(CS10);
	OCR1A   = NB_TICK;
	TIMSK1 |= _BV(OCIE1A);
}

// initialisation des afficheurs


void init_task_afficheurs(void)
{
    init_afficheurs();
    
    tab_task[task_afficheurs].task_id = (uint16_t)afficheurs_main;
    tab_task[task_afficheurs].stack_pointer = 0x700;
    tab_task[task_afficheurs].state = 0x00;
}


void init_rgb_main()  // initialisation complete de la matrice
{
   init_rgb();
   
   tab_task[task_matrice].task_id = (uint16_t)rgb_main();
   tab_task[task_matrice].stack_pointer = 0x500;
   tab_task[task_matrice].state = 0x00;

}

void scheduler(void)
{
    compteur++;
    if (compteur>1) compteur = 0;
    
}

ISR(TIMER1_COMPA_vect)
{  
    
    if(!first)
    {
        SAVE_CONTEXT();
        tab_task[compteur].stack_pointer = SP;
    }
    else first=0;
    
    scheduler();

    SP = tab_task[compteur].stack_pointer;
    if(tab_task[compteur].state == 0x00)
    {
        tab_task[compteur].state = 0x01;
        sei();
        if(compteur == 0) rgb_main();
        else afficheurs_main();
        
    }    
    RESTORE_CONTEXT();
}


int main(void)
{  
    init_timer();
    compteur = 0;
    init_task_afficheurs();
    init_rgb_main();
    
    sei();
    
	while(1)
	{
       
	}

	return 0;
}
